package org.artifactory.repo;

import org.apache.http.Header;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.ProtocolException;
import org.apache.http.RequestLine;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.message.BasicHeader;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.repo.service.RepositoryServiceImpl;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mockito;
import org.testng.annotations.Test;

import java.net.URI;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertTrue;
import static org.testng.Assert.assertFalse;

@Test
public class JFrogRedirectStrategyTest extends ArtifactoryHomeBoundTest {

    private static final int REDIRECT_PERMANENTLY = 308;

    @Test
    void testRedirectedPermanently() throws ProtocolException {
        InternalRepositoryService repositoryService = Mockito.spy(new RepositoryServiceImpl());
        HttpRequest request = mock(HttpRequest.class);
        HttpResponse response = mock(HttpResponse.class);
        StatusLine statusLine = mock(StatusLine.class);
        RequestLine requestLine = mock(RequestLine.class);
        when(statusLine.getStatusCode()).thenReturn(REDIRECT_PERMANENTLY);
        when(response.getStatusLine()).thenReturn(statusLine);
        when(request.getRequestLine()).thenReturn(requestLine);
        when(requestLine.getMethod()).thenReturn("GET");
        JFrogRedirectStrategy jFrogRedirectStrategy = new JFrogRedirectStrategy(repositoryService.getHostsCache());
        assertTrue(jFrogRedirectStrategy.isRedirected(request, response, null));
    }

    @Test
    void testRedirectedPermanentlyFromExternalToLocalhost() throws ProtocolException {
        InternalRepositoryService repositoryService = Mockito.spy(new RepositoryServiceImpl());
        HttpGet request = mock(HttpGet.class);
        HttpResponse response = mock(HttpResponse.class);
        StatusLine statusLine = mock(StatusLine.class);
        RequestLine requestLine = mock(RequestLine.class);
        Header locationHeader = new BasicHeader(HttpHeaders.LOCATION, "http://localhost:8082");
        response.addHeader(locationHeader);
        when(statusLine.getStatusCode()).thenReturn(301);
        when(response.getStatusLine()).thenReturn(statusLine);
        when(request.getRequestLine()).thenReturn(requestLine);
        when(requestLine.getMethod()).thenReturn("GET");
        when(response.getFirstHeader(HttpHeaders.LOCATION)).thenReturn(locationHeader);
        when(request.getURI()).thenReturn(URI.create("http://172.17.0.1:9999"));
        JFrogRedirectStrategy jFrogRedirectStrategy = new JFrogRedirectStrategy(repositoryService.getHostsCache());
        assertFalse(jFrogRedirectStrategy.isRedirected(request, response, null));
    }

    @Test
    void testRedirectedPermanentlyFromLocalhostToLocalhost() throws ProtocolException {
        InternalRepositoryService repositoryService = Mockito.spy(new RepositoryServiceImpl());
        HttpGet request = mock(HttpGet.class);
        HttpResponse response = mock(HttpResponse.class);
        StatusLine statusLine = mock(StatusLine.class);
        RequestLine requestLine = mock(RequestLine.class);
        Header locationHeader = new BasicHeader(HttpHeaders.LOCATION, "http://localhost:8082");
        response.addHeader(locationHeader);
        when(statusLine.getStatusCode()).thenReturn(301);
        when(response.getStatusLine()).thenReturn(statusLine);
        when(request.getRequestLine()).thenReturn(requestLine);
        when(requestLine.getMethod()).thenReturn("GET");
        when(response.getFirstHeader(HttpHeaders.LOCATION)).thenReturn(locationHeader);
        when(request.getURI()).thenReturn(URI.create("http://localhost:1234"));
        JFrogRedirectStrategy jFrogRedirectStrategy = new JFrogRedirectStrategy(repositoryService.getHostsCache());
        assertTrue(jFrogRedirectStrategy.isRedirected(request, response, null));
    }
}