package org.artifactory.repo.cleanup;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.docker.DockerAddon;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.service.InternalRepositoryService;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.times;

/**
 * @author Alexei Vainshtein
 */
public class DockerUploadsFolderCleanupServiceImplTest {

    @Test
    public void testDoCleanup() {
        InternalRepositoryService repositoryService = mock(InternalRepositoryService.class);
        when(repositoryService.getLocalAndFederatedRepoDescriptors()).thenReturn(createLocalAndFederatedRepos());
        AddonsManager addonsManager = mock(AddonsManager.class);
        DockerAddon dockerAddon = mock(DockerAddon.class);
        when(addonsManager.addonByType(DockerAddon.class)).thenReturn(dockerAddon);
        DockerUploadsFolderCleanupServiceImpl dockerUploadsFolderCleanupService =
                new DockerUploadsFolderCleanupServiceImpl(repositoryService, addonsManager, null);
        dockerUploadsFolderCleanupService.doCleanup();
        verify(dockerAddon, times(1)).searchAndCleanupTempFolders("key");
        verify(dockerAddon, times(1)).searchAndCleanupTempFolders("fed");
        verify(dockerAddon, times(0)).searchAndCleanupTempFolders("fed1");
    }

    private List<LocalRepoDescriptor> createLocalAndFederatedRepos() {
        List<LocalRepoDescriptor> repos = new ArrayList<>();
        LocalRepoDescriptor localRepoDescriptor = new LocalRepoDescriptor();
        localRepoDescriptor.setKey("key");
        localRepoDescriptor.setType(RepoType.Docker);
        repos.add(localRepoDescriptor);

        addFederationRepo(repos, "fed", RepoType.Docker);
        addFederationRepo(repos, "fed1", RepoType.Generic);
        return repos;
    }

    private void addFederationRepo(List<LocalRepoDescriptor> repos, String key, RepoType type) {
        FederatedRepoDescriptor fedRepoDesc;
        fedRepoDesc = new FederatedRepoDescriptor();
        fedRepoDesc.setKey(key);
        fedRepoDesc.setType(type);
        repos.add(fedRepoDesc);
    }
}