package org.artifactory.storage.db.init.migration.service;

import com.google.common.collect.Lists;
import org.artifactory.storage.db.init.migration.MarkerMigrationInfoBlob;
import org.artifactory.storage.db.init.migration.dao.MigrationStatusDao;
import org.artifactory.storage.db.init.migration.entity.DbMigrationStatus;
import org.artifactory.storage.db.init.migration.model.MigrationStatus;
import org.jfrog.common.JsonUtils;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoSession;
import org.mockito.quality.Strictness;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.Collection;
import java.util.List;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

/**
 * @author Mor Merhav
 */
public class MigrationStatusStorageServiceImplTest {

    private MigrationStatusStorageService migrationStatusStorageService;
    private MockitoSession mockitoSession;

    @Mock
    private MigrationStatusDao migrationStatusDao;

    @BeforeMethod
    public void setUp() {
        mockitoSession = Mockito.mockitoSession()
                .strictness(Strictness.STRICT_STUBS)
                .initMocks(this)
                .startMocking();

        migrationStatusStorageService = new MigrationStatusStorageServiceImpl(migrationStatusDao);
    }

    @AfterMethod
    public void tearDown() {
        mockitoSession.finishMocking();
    }

    @Test
    public void testFindAllUnfinishedByPrefixWithInfoBlob() throws SQLException {
        String prefix = "prefix";
        MarkerMigrationInfoBlob infoBlob = new MarkerMigrationInfoBlob("Serverid");
        byte[] infoBlobBytes = JsonUtils.getInstance().valueToByteArray(infoBlob);
        DbMigrationStatus dbMigrationStatus = new DbMigrationStatus("id", 1L, 2L, infoBlobBytes);
        Collection<DbMigrationStatus> dbMigrationStatuses = Lists.newArrayList(dbMigrationStatus);
        when(migrationStatusDao.findAllUnfinishedByPrefix(prefix)).thenReturn(dbMigrationStatuses);

        List<MigrationStatus>  migrationStatuses = migrationStatusStorageService
                .findAllUnfinishedByPrefixWithInfoBlob(prefix, MarkerMigrationInfoBlob.class);

        assertNotNull(migrationStatuses);
        assertEquals(migrationStatuses.size(), 1);

        MigrationStatus migrationStatus = migrationStatuses.get(0);
        assertEquals(migrationStatus.getIdentifier(), "id");
        assertEquals(migrationStatus.getStarted(), 1L);
        assertEquals(migrationStatus.getFinished(), 2L);
        assertTrue(migrationStatus.isFinished());
        assertNotNull(migrationStatus.getMigrationInfoBlob());
        assertEquals(migrationStatus.getMigrationInfoBlob().getModel(), MarkerMigrationInfoBlob.class);
        MarkerMigrationInfoBlob actualInfoBlob = (MarkerMigrationInfoBlob)migrationStatus.getMigrationInfoBlob();
        assertEquals(actualInfoBlob, infoBlob);
    }
}