package org.artifactory.rest.resource.plugin;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.HaAddon;
import org.artifactory.addon.plugin.ResponseCtx;
import org.artifactory.addon.rest.RestAddon;
import org.artifactory.aol.AolService;
import org.artifactory.api.build.BuildService;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.http.HttpStatus;

import javax.ws.rs.core.Response;

import static org.fest.assertions.Assertions.assertThat;
import static org.mockito.Mockito.anyBoolean;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;

/**
 * @author Tamir Hadad
 */
@RunWith(MockitoJUnitRunner.class)
public class PluginResourceTest {

    @Mock
    private AddonsManager addonsManager;
    @Mock
    private BuildService buildService;
    @Mock
    private AolService aolService;
    @Mock
    private HaAddon haAddon;
    @Mock
    private RestAddon restAddon;
    @Mock
    private ResponseCtx responseCtx;

    private PluginsResource pluginsResource;
    private AutoCloseable mocks;

    @Before
    public void init() {
        mocks = openMocks(this);
        this.pluginsResource = new PluginsResource(buildService, addonsManager, aolService);
    }

    @After
    public void tearDown() throws Exception {
        mocks.close();
    }

    @Test
    public void testAolNotDashboardUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(false);
        when(addonsManager.addonByType(HaAddon.class)).thenReturn(haAddon);
        when(haAddon.isHaAuthentication()).thenReturn(false);
        Response response = pluginsResource.reload(true);
        assertThat(response.getStatus()).isEqualTo(HttpStatus.FORBIDDEN.value());
    }

    @Test
    public void testAolDashboardUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(true);
        when(addonsManager.addonByType(RestAddon.class)).thenReturn(restAddon);
        when(restAddon.reloadPlugins(anyBoolean())).thenReturn(responseCtx);
        when(responseCtx.getStatus()).thenReturn(HttpStatus.OK.value());
        when(responseCtx.getMessage()).thenReturn("tests");
        Response response = pluginsResource.reload(true);
        assertThat(response.getStatus()).isEqualTo(HttpStatus.OK.value());
    }

    @Test
    public void testAolHaUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(false);
        when(addonsManager.addonByType(HaAddon.class)).thenReturn(haAddon);
        when(haAddon.isHaAuthentication()).thenReturn(true);
        when(addonsManager.addonByType(RestAddon.class)).thenReturn(restAddon);
        when(restAddon.reloadPlugins(anyBoolean())).thenReturn(responseCtx);
        when(responseCtx.getStatus()).thenReturn(HttpStatus.OK.value());
        when(responseCtx.getMessage()).thenReturn("tests");
        Response response = pluginsResource.reload(true);
        assertThat(response.getStatus()).isEqualTo(HttpStatus.OK.value());
    }

    @Test
    public void testNotAol() {
        when(aolService.isAol()).thenReturn(false);
        when(addonsManager.addonByType(RestAddon.class)).thenReturn(restAddon);
        when(restAddon.reloadPlugins(anyBoolean())).thenReturn(responseCtx);
        when(responseCtx.getStatus()).thenReturn(HttpStatus.OK.value());
        when(responseCtx.getMessage()).thenReturn("tests");
        Response response = pluginsResource.reload(true);
        assertThat(response.getStatus()).isEqualTo(HttpStatus.OK.value());
    }
}
